package util

import "sync"

// LockingBoolMap is a lockable map[string]bool
type LockingBoolMap struct {
	sync.RWMutex
	data map[string]bool
}

// NewLockingBoolMap creates a new instance of LockingBoolMap
func NewLockingBoolMap() LockingBoolMap {
	return LockingBoolMap{
		data: make(map[string]bool),
	}
}

// Get will Rlock and return data from a lockingBoolMap
func (m *LockingBoolMap) Get(key string) bool {
	m.RLock()
	defer m.RUnlock()
	return m.data[key]
}

// Set will Lock and set data in a lockingBoolMap
func (m *LockingBoolMap) Set(key string, val bool) {
	m.Lock()
	defer m.Unlock()
	m.data[key] = val
}

// Len will Rlock and return length of a lockingBoolMap
func (m *LockingBoolMap) Len() int {
	m.RLock()
	defer m.RUnlock()
	return len(m.data)
}

// Delete locks and removes a key from LockingBoolMap
func (m *LockingBoolMap) Delete(key string) {
	m.Lock()
	defer m.Unlock()
	delete(m.data, key)
}
