package util

import (
	"strings"

	// Quark
	quarkv1alpha1 "quark.netapp.io/volume-controller/pkg/apis/volumecontroller/v1alpha1"
)

func GetNetAppStoragePoolCondition(
	storagePool *quarkv1alpha1.NetAppStoragePool,
	conditionType quarkv1alpha1.NetAppStoragePoolConditionType,
) *quarkv1alpha1.NetAppStoragePoolCondition {
	return getCondition(storagePool.Status.Conditions, conditionType)
}

func getCondition(
	conditions []quarkv1alpha1.NetAppStoragePoolCondition,
	conditionType quarkv1alpha1.NetAppStoragePoolConditionType,
) *quarkv1alpha1.NetAppStoragePoolCondition {
	var condition *quarkv1alpha1.NetAppStoragePoolCondition
	for _, cond := range conditions {
		if cond.Type == conditionType {
			condition = &cond
			break
		}
	}
	return condition
}

func RemoveStoragePoolConditions(
	oldConditions []quarkv1alpha1.NetAppStoragePoolCondition,
	conditionType quarkv1alpha1.NetAppStoragePoolConditionType,
) []quarkv1alpha1.NetAppStoragePoolCondition {
	newConditions := []quarkv1alpha1.NetAppStoragePoolCondition{}
	for _, condition := range oldConditions {
		if condition.Type != conditionType {
			newConditions = append(newConditions, condition)
		}
	}
	return newConditions
}

func IsErrorConditionPresent(
	conditions []quarkv1alpha1.NetAppStoragePoolCondition,
	conditionType quarkv1alpha1.NetAppStoragePoolConditionType,
) (conditionPresent bool, isRetryError bool) {
	condition := getCondition(conditions, conditionType)

	if condition != nil {
		return true, (condition.ForceRetry || VerifyRetryErrorMessage(condition.Message))
	} else {
		return false, false
	}
}

// If Non-Retry Error conditon present then set the ForceRetry flag so that podrick can treat this like aretry error and can update the dmap
func SetForceRetryInErrorCondition(
	conditions []quarkv1alpha1.NetAppStoragePoolCondition,
	conditionType quarkv1alpha1.NetAppStoragePoolConditionType,
) []quarkv1alpha1.NetAppStoragePoolCondition {
	newConditions := []quarkv1alpha1.NetAppStoragePoolCondition{}
	for _, condition := range conditions {
		if condition.Type == conditionType {
			condition.ForceRetry = true
		}

		newConditions = append(newConditions, condition)
	}
	return newConditions
}

func VerifyRetryErrorMessage(errMsg string) bool {
	return strings.Contains(strings.ToLower(errMsg), "retrying")
}

func SetRetryErrorMessage(errMsg string) string {
	if !VerifyRetryErrorMessage(errMsg) {
		errMsg = "Retrying: " + errMsg
	}
	return errMsg
}

func GetRetryErrorMessage(errMsg string) string {
	if VerifyRetryErrorMessage(errMsg) && strings.Contains(errMsg, ":") {
		_, errMsg, _ = strings.Cut(errMsg, ":")
		errMsg = strings.TrimSpace(errMsg)
	}
	return errMsg
}
